var current_playlist="";
var volume=0;
var digits=[0,0,0,0];
var curr_digit=0;

var alarm_timer=null;
var signal_timer=null;
var key_timer=null;

function init()
{
	if(window.widget)
	{
		current_playlist=widget.preferenceForKey(widget.identifier+".playlist");
		for(var i=0;i<4;i++) change_digit(i,widget.preferenceForKey(widget.identifier+".digit"+i)||0);

		volume=widget.preferenceForKey(widget.identifier+".volume");
		if(volume==null) volume=42;
		document.getElementById("volume_slider").value=volume;
		
		var time=widget.preferenceForKey(widget.identifier+".alarm_time");
		if(time) start_alarm(time);
	}
}


function keypress(e)
{
	if(e.charCode>=48&&e.charCode<=57)
	{
		if(key_timer) clearTimeout(key_timer);
		key_timer=setTimeout(function() {
			key_timer=null;
			curr_digit=0;
		},1500);

		var key=e.charCode-48;

		if(curr_digit==0)
		{
			if(key>2)
			{
				change_digit(0,0,true);
				curr_digit++;
			}
			else if(key==2 && digits[1]>3)
			{
				change_digit(1,3,false);
			}
		}
		else if(curr_digit==1)
		{
			if(digits[0]==2 && key>3) return;
		}
		else if(curr_digit==2)
		{
			if(key>5) return;
		}

		change_digit(curr_digit,key,true);

		curr_digit=(curr_digit+1)%4;
	}
}

function change_digit(n,val,flash)
{
	var img_el=document.getElementById("digit"+n);

	if(flash)
	{
		img_el.src="Images/underscore.png";
		setTimeout(function() { img_el.src="Images/"+val+".png"; },250);
	}
	else img_el.src="Images/"+val+".png";

	digits[n]=val;
	if(window.widget) widget.setPreferenceForKey(val,widget.identifier+".digit"+n);
}

function count_digit(n,dir)
{
	var time=digits[0]*600+digits[1]*60+digits[2]*10+digits[3];
	var mult=[600,60,10,1],mod=[3,10,6,10];

	time+=mult[n]*dir;

	if(time>=1440)
	{
		if(n==0)
		{
			if(time>=1800) time-=1800;
			else time-=1200;
		}
		else time-=1440;
	}

	if(time<0)
	{
		if(n==0)
		{
			if(time<-360) time+=1800;
			else time+=1200;
		}
		else time+=1440;
	}

	for(var i=0;i<4;i++) change_digit(i,Math.floor(time/mult[i])%mod[i],false);
}

function trigger()
{
	if(alarm_timer) stop_alarm();
	else if(signal_timer)
	{
		clearInterval(signal_timer);
		signal_timer=null;

		var button_el=document.getElementById("button");
		button_el.className="unlit";
	}
	else
	{
		set_alarm(digits[0]*10+digits[1],digits[2]*10+digits[3]);
	}
}

function set_alarm(hours,minutes)
{
	var date=new Date();
	var curr_hours=date.getHours();
	var curr_minutes=date.getMinutes();

	if(curr_hours>hours||(curr_hours==hours&&curr_minutes>=minutes))
	date=new Date(date.getTime()+86400000); // tomorrow

	date.setHours(hours,minutes,0,0);

	var time=date.getTime();

	if(window.widget) widget.setPreferenceForKey(time,widget.identifier+".alarm_time");

	start_alarm(time);
}

function start_alarm(time)
{
	var currtime=new Date().getTime();
	var button_el=document.getElementById("button");

	var alarm=function() {
		button_el.style.className="unlit";

		if(window.widget) widget.setPreferenceForKey(0,widget.identifier+".alarm_time");

		set_volume(0);
		play(current_playlist);

		var length=30;
		var ticks=0;
		var from=0,to=volume;
		signal_timer=setInterval(function() {
			ticks++;

			var eights=ticks%8;
			var seconds=ticks/8;

			if(eights==2||eights==4||eights==6) button_el.className="lit";
			else button_el.className="unlit";

			if(seconds>=length)
			{
				set_volume(to);
				clearInterval(signal_timer);
				signal_timer==null;
			}
			else if(eights==0) set_volume(Math.floor(from+(to-from)*seconds/length));
		},125);

		alarm_timer=null;
	}

	if(time<currtime) alarm();
	else
	{
		alarm_timer=setTimeout(alarm,time-currtime);

		button_el.className="lit";
	}
}

function stop_alarm()
{
	var button_el=document.getElementById("button");
	button_el.className="unlit";

	if(window.widget) widget.setPreferenceForKey(0,widget.identifier+".alarm_time");

	clearTimeout(alarm_timer);
	alarm_timer=null;
}



// iTunes communication

function play(playlist)
{
	if(playlist) widget.system("/usr/bin/osascript -e 'tell application \"itunes\" to play playlist "+encode_unicode(playlist)+"'",null);
	else widget.system("/usr/bin/osascript -e 'tell application \"itunes\" to play'",null);
}

function set_volume(vol)
{
	widget.system("/usr/bin/osascript -e 'tell application \"itunes\" to set sound volume to "+vol+"'",null);
}

function get_playlists()
{
	var list=widget.system("osascript -e 'set text item delimiters to \"\\n\"' -e 'tell application \"itunes\" to return name of playlists as string'",null).outputString;
	return list.replace(/(^\n+|\n+$)/g,"").split(/\n/);
}

function encode_unicode(str)
{
	var encode=false;
	var data="";
	for(var i=0;i<str.length;i++)
	{
		var c=str.charCodeAt(i);
		if(c>128) encode=true;
		var hex=c.toString(16);
		while(hex.length<4) hex="0"+hex;
		data+=hex;
	}

	if(encode) return "(run script ascii character 199&\"data utxt"+data+"\"&ascii character 200)";
	else return '"'+str+'"';
}




// prefs

function change_playlist(el)
{
	current_playlist=el.options[el.selectedIndex].value;
	if(window.widget) widget.setPreferenceForKey(current_playlist,widget.identifier+".playlist");
}

function change_volume(el)
{
	volume=el.value;
	if(window.widget) widget.setPreferenceForKey(volume,widget.identifier+".volume");
}

function show_prefs()
{
	var front=document.getElementById("front");
	var back=document.getElementById("back");

	if(window.widget) widget.prepareForTransition("ToBack");

	front.style.display="none";
	back.style.display="block";

	init_prefs();
	
	if(window.widget) setTimeout('widget.performTransition()',0);

	document.getElementById('fliprollie').style.display='none';
}

function init_prefs()
{
	var playlists=get_playlists();
	var select_el=document.getElementById("playlist_select");

	while(select_el.firstChild) select_el.removeChild(select_el.firstChild);

	var option_el=document.createElement("option");
	option_el.appendChild(document.createTextNode("Current"));
	option_el.setAttribute("value","");
	if(!current_playlist) option_el.setAttribute("selected","selected");
	select_el.appendChild(option_el);

	for(var i=0;i<playlists.length;i++)
	{
		var option_el=document.createElement("option");
		option_el.appendChild(document.createTextNode(playlists[i]));
		option_el.setAttribute("value",playlists[i]);
		if(current_playlist==playlists[i]) option_el.setAttribute("selected","selected");
		select_el.appendChild(option_el);
	}
}

function hide_prefs()
{
	var front=document.getElementById("front");
	var back=document.getElementById("back");

	if(window.widget) widget.prepareForTransition("ToFront");

	back.style.display="none";
	front.style.display="block";
	
	if(window.widget) setTimeout('widget.performTransition()',0);
}



// PREFERENCE BUTTON ANIMATION (- the pref flipper fade in/out)

var flipShown = false;		// a flag used to signify if the flipper is currently shown or not.


// A structure that holds information that is needed for the animation to run.
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};


// mousemove() is the event handle assigned to the onmousemove property on the front div of the widget. 
// It is triggered whenever a mouse is moved within the bounds of your widget.  It prepares the
// preference flipper fade and then calls animate() to performs the animation.

function mousemove (event)
{
	if (!flipShown)			// if the preferences flipper is not already showing...
	{
		if (animation.timer != null)			// reset the animation timer value, in case a value was left behind
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13; 		// set it back one frame
		
		animation.duration = 500;												// animation time, in ms
		animation.starttime = starttime;										// specify the start time
		animation.firstElement = document.getElementById ('flip');		// specify the element to fade
		animation.timer = setInterval ("animate();", 13);						// set the animation function
		animation.from = animation.now;											// beginning opacity (not ness. 0)
		animation.to = 1.0;														// final opacity
		animate();																// begin animation
		flipShown = true;														// mark the flipper as animated
	}
}

// mouseexit() is the opposite of mousemove() in that it preps the preferences flipper
// to disappear.  It adds the appropriate values to the animation data structure and sets the animation in motion.

function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the flip widget
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13;
		
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}


// animate() performs the fade animation for the preferences flipper. It uses the opacity CSS property to simulate a fade.

function animate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
		
	
	T = limit_3(time-animation.starttime, 0, animation.duration);
	
	if (T >= animation.duration)
	{
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}
	
	animation.firstElement.style.opacity = animation.now;
}


// these functions are utilities used by animate()

function limit_3 (a, b, c)
{
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease)
{
    return from + (to - from) * ease;
}

// these functions are called when the info button itself receives onmouseover and onmouseout events

function enterflip(event)
{
	document.getElementById('fliprollie').style.display = 'block';
}

function exitflip(event)
{
	document.getElementById('fliprollie').style.display = 'none';
}